<?php

/**
* Clase que enmascara la conexion con el SGBD PostgreSQL
*
* Esta clase sirve para crear una capa de abstraccion con la base de datos
* de forma que solo se deberia de reescribir esta clase si se decide cambiar 
* de Sistema de gestion de base de datos.
*
* @package postgres.php
* @copyright 2007 2008 Luis Jorge Martin Aznar
* @version 1.0
* @author Luis Jorge Martin Aznar
* creacion= 12-octubre-2007
* modificacion= 14-abril-2008
*
* generator= gedit
*
* This file is part of PreAdmin ©.
*
*    This program is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program.  If not, see <http://www.gnu.org/licenses/>.  
*/

class bd
{
	var $separador;
	var $conecto;
	var $consulta;
	var $result;

/**
* Constructor de la clase
*
* Esta funcion inicializa la clase asignando a la variable separador
* el caracter ¬.
*/
          function bd ()
          {
              $this->separador = "¬";
          }
/**
* Establece la conexion con la base de datos
*
* Esta funcion estable la conexion con la base de datos. 
* Los parametros necesarios los "toma" del fichero de configuracion
* por lo que no es necesario pasarle ningun parametro. 
* @return Un string. Si la conexion se ha establecido el valor del string es
* "conectado" si ha fallador retorna 0.
*/
          function conectar ()
          {
              // conecta a la bbdd
		require(''.dirname(__FILE__).'/../config.php');
		$this-> conexion = $con_postgres=pg_connect("host=$servidor port=5432 dbname=$bd password=$clave_postgre user=$usuario_postgre");
		if ($this-> conexion!='')
		{return ("conectado");
		}else{
		return(0);
		};
          }

/**
* Establece la conexion con el SGBD para manejo de bases de datos 
*
* Esta funcion estable la conexion con la base de datos. 
* Los parametros necesarios 
* por lo que no es necesario pasarle ningun parametro. 
* @return Un string. Si la conexion se ha establecido el valor del string es
* "conectado" si ha fallador retorna 0.
*/
          function conectar2 ($servidor,$clave_postgre,$usuario_postgre)
          {
              // conecta a la bbdd
		$this-> conexion = $con_postgres=pg_connect("host=$servidor port=5432 password=$clave_postgre user=$usuario_postgre");
		if ($this-> conexion!=''){return ("conectado");}else{return(0);};
          }


/**
* Sirve para inicializar la transaccion y define las caracteristicas clave de la misma.
*
* Esta funcion inicializa la transaccion con la base de datos. 
* No tiene parametros de entrada porque siempre inicializaremos la conexion
* en modo lectura-escritura. Todas las acciones sobre la base de datos quedan pendientes
* de que no se produzcan errores en la transaccion. No retorna ningun resultado.
*/
          function begin ()
          {
		$this-> query("set transaction read write");
		$this-> query("begin work");
	  }
/**
* Finaliza la transaccion actual salvando los cambios en la base de datos.
*
* Esta funcion representa el punto del tiempo donde el usuario ha realizado una serie de 
* cambios logicos que se han agrupado y que dado que no ha habido errores 
* se van a salvar los cambios. Posteriormente se cierra la conexion con la base de datos
* No tiene parametros de entrada No retorna ningun resultado.
*/
          function commit ()
          {
		$this-> query("commit work");
		//$this-> cerrar();
	  }

/**
* Funcion para realizar acciones "sql" sobre la base de datos que no retornen resultados.
*
* La usaremos para crear la base de datos sobre la base de datos que
* no generen resultado.
* @param . La instruccion en lenguaje SQL.
* @return Un string. El valor del string es el del Id de la conexion.
*/

          function query ($sql)
          {
		$this-> consulta = pg_query($sql);
		return ($this-> consulta);
	  }

/**
* Funcion para realizar acciones "sql" sobre la base de datos que retornen resultados.
*
* La usaremos para realizar acciones (select,..) sobre la base de datos que
* generen resultado.
* @param . La instruccion en lenguaje SQL.
* @return Un array. El array es de dos dimensiones: La primera se corresponde con el numero de
* fila, y la segunda es un indice de tipo asociativo que se corresponde con el nombre de los campos
* de la tabla o vista sobre la que se ha ejecutado la consulta.
*/
          function query_c ($sql)
          {
		$records = array();
		$this-> consulta = pg_query($sql);
		$rows = pg_num_rows($this-> consulta);
		// si hay filas
       		if ($rows > 0)
       		{
              	 // pasar por todas las entradas del resultado del query
              	for ($i=0; $i<$rows; $i++) 
               	{
                  $records[$i]= pg_fetch_array($this-> consulta, $i);
               	}
       		}else{
		$records[0]="";
		}
		return $records;
	  }
/**
* Funcion para crear el historico en la tabla  "historial".
*
* La usaremos siempre que queramos, para una tabla dada, que se genere un historial de los cambios
* que se produzcan -en la tabla historial-. Crea una funcion y su correspondiente disparador en la 
* base de datos en lenguaje plpgsql siguiendo la estructura de la tabla destino de los datos (tabla historial).
* @param . De tipo string, es el nombre de la tabla sobre la que queremos guardar los cambios.
* @param . Es el campo Primary Key de la tabla sobre la que queremos guardar los cambios.
* @param . Es el campo updatedby_XX de la tabla sobre la que queremos guardar los cambios.
* No retorna ningun resultado.
*/
	  function respaldo($nombretabla,$campoid, $updatedby)
	  {

			$nombretabla=$nombretabla;
			$crea_funcion=pg_query("SELECT column_name FROM information_schema.columns WHERE table_name = '$nombretabla'");
			$n_doc = (pg_num_rows($crea_funcion));
			$sql="CREATE OR REPLACE FUNCTION $nombretabla() RETURNS trigger AS '
			begin ";
			for($m=0;$m<($n_doc);$m++)
			{
				$datos = pg_fetch_array($crea_funcion,$m);
				$campo=$datos['column_name'];
			$sql=$sql."
				if (old.$campo != new.$campo) then
					insert into historial(created,createdby,tabla,campo,id,oldvalue,newvalue)
					values(
					now(),
					new.$updatedby,
					''$nombretabla'',
					''$campo'',
					old.$campoid,
					old.$campo,
					new.$campo
					);
				end if;";
			}
			$sql=$sql."
			return null;
			end;
			' LANGUAGE 'plpgsql'";
			$crea_tabla=pg_query($sql);

			$crea_tabla=pg_query("CREATE TRIGGER $nombretabla
		  	AFTER UPDATE
  			ON $nombretabla
		  	FOR EACH ROW
		  	EXECUTE PROCEDURE $nombretabla();
			");

	  }

/**
* Funcion para crear la función PL/pgSQL para crear el historico de borrados en la tabla  "historial".
*
* La usaremos siempre que queramos, para una tabla dada, que se genere un historial de los cambios
* que se produzcan -en la tabla historial-. Crea una funcion y su correspondiente disparador en la 
* base de datos en lenguaje plpgsql siguiendo la estructura de la tabla destino de los datos (tabla historial).
* @param . De tipo string, es el nombre de la tabla sobre la que queremos guardar los cambios.
* @param . Es el campo Primary Key de la tabla sobre la que queremos guardar los cambios.
* @param . Es el campo updatedby_XX de la tabla sobre la que queremos guardar los cambios.
* No retorna ningun resultado.
*/

	  function respaldo_borrar($nombretabla,$campoid, $updatedby)
	  {

			$nombretabla1=$nombretabla.'_del';
			$crea_funcion=pg_query("SELECT column_name FROM information_schema.columns WHERE table_name = '$nombretabla'");
			$n_doc = (pg_num_rows($crea_funcion));
			$sql="CREATE OR REPLACE FUNCTION $nombretabla1() RETURNS trigger AS '
			begin ";
			for($m=0;$m<($n_doc);$m++)
			{
				$datos = pg_fetch_array($crea_funcion,$m);
				$campo=$datos['column_name'];
			$sql=$sql."
					insert into historial(created,createdby,tabla,campo,id,oldvalue,newvalue)
					values(
					now(),
					old.$updatedby,
					''$nombretabla'',
					''$campo'',
					old.$campoid,
					old.$campo,
					''borrado''
					);
				";
			}
			$sql=$sql."
			return null;
			end;
			' LANGUAGE 'plpgsql'";
			$crea_tabla=pg_query($sql);

			$crea_tabla=pg_query("CREATE TRIGGER $nombretabla1
		  	AFTER DELETE
  			ON $nombretabla
		  	FOR EACH ROW
		  	EXECUTE PROCEDURE $nombretabla1();
			");

	  }
/**
* Funcion para limpiar la base de datos.
*
* @param no tiene parámetros de entrada
*/
function vacumm()
{
	$conexion=pg_query("vacuum full verbose analyze");
	if($conexion!="")//nos aseguramos que hay coincidencia
	{
		$result = $conexion;
	}else{
		$result = '0';
	}
	return $result;
unset ($conecto,$result);
}
/**
* Funcion para reindexar la base de datos.
*
* @param $bd es de tipo string y se corresponde con el nombre de la base de datos
*/
function reindex($bd)
{

	$conexion=pg_query("reindex database $bd");
	if($conexion!="")//nos aseguramos que hay coincidencia
	{
		$result = $conexion;
	}else{
		$result = '0';
	}
	return $result;
unset ($conecto,$result);
}



/**
* Funcion para cerrar la conexion con la base de datos.
*
* La usaremos despues de finalizar la transaccion para liberar la conexion con la base de datos
* y evitar consumir recursos innecesariamente.
* No retorna ningun resultado.
*/
	  function cerrar()
	  {
          // cierra la conexion existente
          pg_close ($this-> conexion);
	 }
} // fin de la clase
?>
